@@ -17,7 +17,7 @@ import string
 from datetime import datetime, timedelta
 
 sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
-from gallery_dl import extractor  # noqa E402
+from gallery_dl import extractor, util  # noqa E402
 from gallery_dl.extractor import mastodon  # noqa E402
 from gallery_dl.extractor.common import Extractor, Message  # noqa E402
 from gallery_dl.extractor.directlink import DirectlinkExtractor  # noqa E402
@@ -25,7 +25,11 @@ from gallery_dl.extractor.directlink import DirectlinkExtractor  # noqa E402
 _list_classes = extractor._list_classes
 
 try:
-    from test import results
+    RESULTS = os.environ.get("GDL_TEST_RESULTS")
+    if RESULTS:
+        results = util.import_file(RESULTS)
+    else:
+        from test import results
 except ImportError:
     results = None
 
@@ -45,7 +49,7 @@ class TestExtractorModule(unittest.TestCase):
         "https://example.org/file.jpg",
         "tumblr:foobar",
         "oauth:flickr",
-        "test:pixiv:",
+        "generic:https://example.org/",
         "recursive:https://example.org/document.html",
     )
 
@@ -109,6 +113,7 @@ class TestExtractorModule(unittest.TestCase):
                     print("Skipping '{}' category checks".format(cat))
                     continue
                 raise
+            self.assertTrue(extr, url)
             self.assertEqual(extr.category, cat, url)
             self.assertEqual(extr.subcategory, sub, url)
             self.assertEqual(extr.basecategory, base, url)
@@ -208,7 +213,7 @@ class TestExtractorModule(unittest.TestCase):
 class TestExtractorWait(unittest.TestCase):
 
     def test_wait_seconds(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         seconds = 5
         until = time.time() + seconds
 
@@ -222,7 +227,7 @@ class TestExtractorWait(unittest.TestCase):
             self._assert_isotime(calls[0][1][1], until)
 
     def test_wait_until(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         until = time.time() + 5
 
         with patch("time.sleep") as sleep, patch.object(extr, "log") as log:
@@ -237,7 +242,7 @@ class TestExtractorWait(unittest.TestCase):
             self._assert_isotime(calls[0][1][1], until)
 
     def test_wait_until_datetime(self):
-        extr = extractor.find("test:")
+        extr = extractor.find("generic:https://example.org/")
         until = datetime.utcnow() + timedelta(seconds=5)
         until_local = datetime.now() + timedelta(seconds=5)
 
